import { TodaClient } from "../src/client/client.js";
import { LocalInventoryClient } from "../src/client/inventory.js";
import { RemoteRelayClient } from "../src/client/relay.js";
import { SECP256r1 } from "../src/client/secp256r1.js";
import { Hash } from "../src/core/hash.js";
import { Twist } from "../src/core/twist.js";

import fs from 'fs-extra';
import path from 'path';

async function kpFromDisk(secp, privateKeyPath, publicKeyPath) {
    publicKeyPath = publicKeyPath || privateKeyPath + ".pub";

    return new secp(await secp.importKey("pkcs8",
                        fs.readFileSync(privateKeyPath)),
                    await secp.importKey("spki",
                        fs.readFileSync(publicKeyPath)));
}

async function foo() {
    const f = "C:/Users/csulp/Downloads/sender/41f36676390e3597df35c9d216be45a9.twin.todaq.net_tmp/store";
    const now = Date.now();
    const mem = process.memoryUsage();

    const i = new LocalInventoryClient(f);

    console.log(`t = ${Date.now() - now}`);
    console.log("Pre load:", JSON.stringify(mem));
    console.log("Post load:", JSON.stringify(process.memoryUsage()));
    const c = new TodaClient(i, "https://cdn.relay.todaq.net/next");
    const kp = await kpFromDisk(SECP256r1, path.join("C:/Users/csulp/Downloads/sender/41f36676390e3597df35c9d216be45a9.twin.todaq.net_tmp", "secure/id_secp256r1"));
    c.addSatisfier(kp);

    c.defaultRelayUrl = "https://relay.todaq.net/api/hoist";
    c.shieldSalt = "c:/Users/csulp/Downloads/sender/41f36676390e3597df35c9d216be45a9.twin.todaq.net_tmp/secure/salt";
    c.defaultRelayHash = Hash.fromHex("41f36676390e3597df35c9d216be45a982f924e4788a859e42b7831084a3d7ac23");
    c.defaultTopLineHash = Hash.fromHex("41474fbd551847d1122ed4398464830d9f139cbdc8de10aa40aa54824db1dd1cc4");

    await c.archiveUnownedFiles();
}

foo();