import { Atoms } from "../src/core/atoms.js";
import { ByteArray } from "../src/core/byte-array.js";
import { Hash, Sha256 } from "../src/core/hash.js";
import { ArbitraryPacket, Packet } from "../src/core/packet.js";

function createMockPacket(x) {
    const b = new ByteArray(30000000);
    for (let i = 0; i < b.length; i += 7) {
        b[i] = 255;
    }
    for (let i = 0; i < b.length; i += 11) {
        b[i] = x;
    }
    const p = new ArbitraryPacket(b);
    const h = Sha256.fromPacket(p);
    const out = new ByteArray(h.numBytes() + p.getLength());
    out.set(h.toBytes(), 0);
    out.set(p.toBytes(), h.numBytes());
    return out;
}

function createMockPackets(...xs) {
    let b = new ByteArray(0);
    for (const x of xs) {
        b = b.concat(createMockPacket(x));
    }
    return b;
}

function createMockAtoms(...xs) {
    return Atoms.fromBytes(createMockPackets(...xs));
}

function printMem(moniker) {
    if (moniker) {
        console.log(moniker, JSON.stringify(process.memoryUsage()));
    } else {
        console.log(JSON.stringify(process.memoryUsage()));
    }
}

async function foo() {
    printMem();
    const t = Date.now();
    const a = createMockAtoms(1, 2);
    const b = createMockAtoms(2, 3);
    console.log(Date.now() - t);
    printMem();
    await new Promise(r => setTimeout(r, 5000));
    printMem();
}

foo();