import { Twist, TwistBuilder } from "./core/twist.js";
import { Hash, Sha256 } from "./core/hash.js";
import { ArbitraryPacket } from "./core/packet.js";
import { ByteArray } from "./core/byte-array.js";
import { Atoms } from "./core/atoms.js";
import fs from "fs";
import { HashMap } from "./core/map.js";
import { v4 as uuid } from 'uuid';

function randH() {
    let tetherStr = uuid();
    let tetherBytes = new ByteArray(new TextEncoder().encode(tetherStr));
    return new Sha256(Sha256.hash(tetherBytes));
}

function newTwist(obj, tetherH) {
    let tb = new TwistBuilder();
    let fullPrev = obj.fullTwists.slice(-1)[0]; // Full history + shields
    let partialPrev = obj.partialTwists.slice(-1)[0]; // No shield info, no prevs
    let prevH = fullPrev.getHash();
    tb.setPrevHash(prevH);
    let twist;
    let shield;
    if (tetherH) {
        let shieldStr = uuid();
        let shieldBytes = new ByteArray(new TextEncoder().encode(shieldStr));
        shield = new ArbitraryPacket(shieldBytes);
        tb.setShield(shield);
        tb.setTetherHash(tetherH);
    }
    twist = tb.twist();
    if (shield) {
        obj.shields.set(twist.getHash(), shield);
    }
    // Save the FULL twist
    let full = new Twist(twist.getAtoms(), twist.getHash());
    full.safeAddAtoms(fullPrev.getAtoms());
    obj.fullTwists.push(full);

    // Save the PARTIAL twist (ie, NO SHIELD, NO PREVS)
    let partial = new Twist(twist.getAtoms(), twist.getHash());
    if (shield) {
        partial.getAtoms().delete(partial.getBody().getShieldHash());
    }
    obj.partialTwists.push(partial);
    
    // Save the NEXT file
    let next = new Twist(partial.getAtoms(), twist.getHash());
    next.safeAddAtoms(partialPrev.getAtoms());
    obj.nexts.set(prevH, next);

    obj.hashes.push(twist.getHash());
}

function makeTwists(...tethers) {
    let t0 = new TwistBuilder();
    t0.setPrevHash(randH());
    t0 = t0.twist();

    let hashes = [t0.getHash()];
    let fullTwists = [t0];
    let partialTwists = [t0];
    let shields = new HashMap();
    let nexts = new HashMap();

    let obj = {fullTwists,
               partialTwists,
               shields,
               nexts, 
               hashes};

    tethers.forEach(x => {
        newTwist(obj, x);
    });

    saveInfo(obj);

    return hashes;
}

function saveShields(obj) {
    obj.shields.forEach((shield, twistH) => {
        fs.writeFileSync("C:\\Users\\csulp\\todaadot\\test\\client\\remoteNextRelay_files\\" + twistH + ".shield", shield.serialize());
    });
}

function saveNexts(obj) {
    obj.nexts.forEach((twist, prevH) => {
        fs.writeFileSync("C:\\Users\\csulp\\todaadot\\test\\client\\remoteNextRelay_files\\" + prevH + ".next.toda", twist.getAtoms().toBytes());
    });
}

function saveTwists(obj) {
    obj.fullTwists.forEach((twist) => {
        fs.writeFileSync("C:\\Users\\csulp\\todaadot\\test\\client\\remoteNextRelay_files\\" + twist.getHash() + ".toda", twist.getAtoms().toBytes());
    });
}

function saveInfo(obj) {
    saveShields(obj);
    saveNexts(obj);
    saveTwists(obj);
}

let topHashes = makeTwists(false, randH(), false, false, randH(), false, randH(), false);
console.log(topHashes.join("\n"));