import { TodaClient } from "../src/client/client.js";
import { LocalInventoryClient } from "../src/client/inventory.js";
import { v4 as uuid } from 'uuid';
import { ByteArray } from "../src/core/byte-array.js";
import { uuidCargo } from "../test/util.js";
import { Hash } from "../src/core/hash.js";
import { Shield } from "../src/core/shield.js";

const path = "C:/Users/csulp/Downloads/testes";
const topline = Hash.fromHex("417e41bf0b449cfa7babd0eb05b863a4905c2ebd41c7cb4181edd2cf5caa7d79d1");
const fileServer = "https://cdn.dev.todaq.lol/next";
const hoistServer = "https://dev.todaq.lol/api/hoist";

async function setUp() {
    const aliceInv = new LocalInventoryClient(path + "/" + uuid());
    const alice = new TodaClient(aliceInv, fileServer);
    alice._getSalt = () => ByteArray.fromHex("");
    alice.defaultTopLineHash = topline;
    alice.defaultRelayUrl = hoistServer;

    const bobInv = new LocalInventoryClient(path + "/" + uuid());
    const bob = new TodaClient(bobInv, fileServer);
    bob._getSalt = () => ByteArray.fromHex("");
    bob.defaultTopLineHash = topline;
    bob.defaultRelayUrl = hoistServer;

    const a0 = await alice.create(topline, null, uuidCargo());
    const a1 = await alice.append(a0, topline);

    alice.defaultRelayHash = a0.getHash();

    const b0 = await bob.create(topline, null, uuidCargo());
    const b1 = await bob.append(b0, topline);

    bob.defaultRelayHash = b0.getHash();

    console.log("WOW");

    const {root} = await alice.mint(99999, 1, a1.getHash());

    alice.address = a1.getHash();
    alice.moniker = "Alice";
    bob.address = b1.getHash();
    bob.moniker = "Bob";

    await transfer(alice, bob, root, 5000);

    return {alice, bob, root};
}

async function transfer(sender, receiver, typeHash, amount) {
    function log(s) {
        console.log(`${sender.moniker}: ${s}`);
    }
    log(`Sending`);
    const twists = await sender.transfer({
        amount, typeHash, destHash: receiver.address
    });
    log(`Putting ${JSON.stringify(twists.map(t => t.getHash()))}`);
    for (const t of twists) {
        await receiver.put(t);
    }
    sender.getBalance(typeHash, true);
    receiver.getBalance(typeHash, true);

    const bal0 = await sender.getBalance(typeHash);
    const bal1 = await receiver.getBalance(typeHash);

    log(JSON.stringify([bal0, bal1]));
}

async function repeatedly(sender, receiver, typeHash, times) {
    for (let i = 0; i < times; i++) {
        await transfer(sender, receiver, typeHash, 15);
    }
}

async function foo() {
    const { alice, bob, root } = await setUp();
    repeatedly(alice, bob, root, 50);
    repeatedly(bob, alice, root, 50);
}

foo();