/*************************************************************
 * TODAQ Open: TODA File Implementation
 * Toronto 2022
 *
 * Apache License 2.0
 *************************************************************/

import { RequirementList, RequirementSatisfier, SatisfactionList } from "../core/reqsat.js";
import { MissingHashPacketError } from "../core/twist.js";

class RSList extends RequirementSatisfier {
    static requirementTypeHash = RequirementList.REQ_LIST;

    /**
     * @param {TodaClient} todaClient
     */
    constructor(todaClient) {
        super();
        this.todaClient = todaClient;
    }

    static verifySatisfaction(reqTypeHash, twist, reqPacket, satPacket) {
        const unpackedReq = RequirementList.unpack(twist.getAtoms(), 
                                                   reqPacket);
        const unpackedSat = satPacket.getShapedValue();
        if (!unpackedReq ||
            !unpackedSat || 
            unpackedReq.length != unpackedSat.length) {
            return false;
        }
        let sum = 0;
        for (const i in unpackedReq) {
            const {weight, requirementHash} = unpackedReq[i];
            const req = twist.get(requirementHash);
            const satHash = unpackedSat[i];
            const sat = twist.get(satHash);
            if (!weight || !req || !sat) {
                continue;
            }
            if (RequirementSatisfier.verifyAll(twist, req, sat)) {
                sum += weight;
            }
        }
        return sum >= 255;
    }

    isSatisfiable(twist, reqPacket) {
        const unpackedReq = RequirementList.unpack(twist.getAtoms(), reqPacket);
        let sum = 0;
        for (const i in unpackedReq) {
            const {weight, requirementHash} = unpackedReq[i];
            const req = twist.get(requirementHash);
            if (!weight || !req) {
                continue;
            }
            const satisifers = this.todaClient.requirementSatisfiers;
            if (RequirementSatisfier.allSatisfiable(twist, req)) {
                sum += weight;
            }
        }
        return sum >= 255;
    }
}

/**
class SECP256r1 extends LocalKeyPair {
    static requirementTypeHash = SignatureRequirement.REQ_SECP256r1;

    static curve = {
        name: "ECDSA",
        namedCurve: "P-256"
    };
    static sigParams = {
        name: "ECDSA",
        hash: { name: "SHA-256" }
    };

    
      * @param privateKey <CryptoKey>
      * @param publicKey <CryptoKey>
      
    constructor(privateKey, publicKey) {
        super();
        this.privateKey = privateKey;
        this.publicKey = publicKey;
    }

    static importRawPubKey(keyFormat, buffer) {
        // NB: webcrypto will subtly throw if you import a pubkey with "sign" usage
        return crypto.subtle.importKey(
            keyFormat,
            buffer,
            this.curve,
            true, // extractable
            ["verify"]
        );
    }

    static importKey(keyFormat, buffer) {
        let b = this._fromPEM(bytesToUtf8(buffer));
        return crypto.subtle.importKey(
            keyFormat,
            b,
            this.curve,
            true, // extractable
            ["sign", "verify"]
        );
    }

    static async generate() {
        let keyPair = await crypto.subtle.generateKey(
            this.curve,
            true, // extractable
            ["sign", "verify"] //can be any combination of "sign" and "verify"
        );
        return new SECP256r1(keyPair.privateKey, keyPair.publicKey);
    }

    async signBytes(bytes) {
        let signedString = await crypto.subtle.sign(
            SECP256r1.sigParams,
            this.privateKey,
            bytes
        );

        return LocalKeyPair._toDER(signedString);
    }

    static async verifySig(rawPublicKeyBuffer, derSignatureBuffer, data) {
        let signedString = this._fromDER(derSignatureBuffer);
        let pubKey = await this.importRawPubKey("spki", rawPublicKeyBuffer);
        return crypto.subtle.verify(
            this.sigParams,
            pubKey,
            signedString,
            data
        );
    }
}
**/

//RequirementSatisfier.registerSatisfier(
//    SECP256r1.requirementTypeHash, SECP256r1);

//export { SECP256r1 };